@@ -1,7 +1,6 @@
-"""List partitioning library.
-
-Minimal Python library that provides common functions
-related to partitioning lists.
+"""
+Minimal library that enables partitioning of
+iterable objects in a concise manner.
 """
 
 import doctest
@@ -48,142 +47,159 @@ def _slice(xs, lower, upper):
         except:
             raise TypeError('object does not support retrieval of slices') from None
 
-def parts(xs, number=None, length=None):
+def parts(xs, number=None, length=None): # pylint: disable=R0912,R0915
     """
-    Split a list into either the specified number of parts or
-    a number of parts each of the specified length. The elements
-    are distributed somewhat evenly among the parts if possible.
+    This function splits an iterable object into either the specified
+    number of parts or a number of parts each of the specified length.
+    When input parameters lead to ambiguous or conflicting constraints,
+    either elements are distributed in a best-effort manner or an
+    exception is raised (depending on the specific scenario).
 
-    >>> list(parts([1,2,3,4,5,6,7], length=1))
-    [[1], [2], [3], [4], [5], [6], [7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=2))
-    [[1, 2], [3, 4], [5, 6], [7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=3))
-    [[1, 2, 3], [4, 5, 6], [7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=4))
-    [[1, 2, 3, 4], [5, 6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=5))
-    [[1, 2, 3, 4, 5], [6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=6))
-    [[1, 2, 3, 4, 5, 6], [7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=7))
+    In the simplest case, the target number of parts can be specified.
+
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 1))
     [[1, 2, 3, 4, 5, 6, 7]]
-
-    >>> list(map(list, parts(iter([1,2,3,4,5,6,7]), length=4)))
-    [[1, 2, 3, 4], [5, 6, 7]]
-
-    >>> list(parts([1,2,3,4,5,6,7], 1))
-    [[1, 2, 3, 4, 5, 6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 2))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 2))
     [[1, 2, 3], [4, 5, 6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 3))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 3))
     [[1, 2], [3, 4], [5, 6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 4))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 4))
     [[1], [2, 3], [4, 5], [6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 5))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 5))
     [[1], [2], [3], [4, 5], [6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 6))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 6))
     [[1], [2], [3], [4], [5], [6, 7]]
-    >>> list(parts([1,2,3,4,5,6,7], 7))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 7))
     [[1], [2], [3], [4], [5], [6], [7]]
 
-    >>> list(parts([1,2,3,4,5,6,7], 7, [1,1,1,1,1,1,1]))
+    The target length for each part can be specified; the number of parts will
+    be determined based on the length and the available number of items.
+
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=1))
     [[1], [2], [3], [4], [5], [6], [7]]
-    >>> list(parts([1,2,3,4,5,6,7], length=[1,1,1,1,1,1,1]))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=2))
+    [[1, 2], [3, 4], [5, 6], [7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=3))
+    [[1, 2, 3], [4, 5, 6], [7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=4))
+    [[1, 2, 3, 4], [5, 6, 7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=5))
+    [[1, 2, 3, 4, 5], [6, 7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=6))
+    [[1, 2, 3, 4, 5, 6], [7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=7))
+    [[1, 2, 3, 4, 5, 6, 7]]
+    >>> list(map(list, parts(iter([1, 2, 3, 4, 5, 6, 7]), length=4)))
+    [[1, 2, 3, 4], [5, 6, 7]]
+
+    A sequence of length values can be specified. The entry
+    at each position in the sequence of lengths dictates the
+    length of the part in the corresponding position in the
+    output.
+
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], 7, [1, 1, 1, 1, 1, 1, 1]))
     [[1], [2], [3], [4], [5], [6], [7]]
-    >>> list(parts([1,2,3,4,5,6], length=[2,2,2]))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], length=[1, 1, 1, 1, 1, 1, 1]))
+    [[1], [2], [3], [4], [5], [6], [7]]
+    >>> list(parts([1, 2, 3, 4, 5, 6], length=[2, 2, 2]))
     [[1, 2], [3, 4], [5, 6]]
-    >>> list(parts([1,2,3,4,5,6], length=[1,2,3]))
+    >>> list(parts([1, 2, 3, 4, 5, 6], length=[1, 2, 3]))
     [[1], [2, 3], [4, 5, 6]]
 
-    >>> list(parts([1,2,3,4,5,6], 2, 3))
+    A descriptive exception is raised when parameter values cannot
+    be satisfied, cause a conflict, or have an incorrect type.
+
+    >>> list(parts([1, 2, 3, 4, 5, 6], 2, 3))
     [[1, 2, 3], [4, 5, 6]]
-    >>> list(parts([1,2,3,4,5,6], number=3, length=2))
+    >>> list(parts([1, 2, 3, 4, 5, 6], number=3, length=2))
     [[1, 2], [3, 4], [5, 6]]
-    >>> list(parts(iter([1,2,3,4,5,6]), number=3, length=2)) # doctest: +NORMALIZE_WHITESPACE
+    >>> list(parts(iter([1, 2, 3, 4, 5, 6]), number=3, length=2)) # doctest: +NORMALIZE_WHITESPACE
     Traceback (most recent call last):
       ...
     TypeError: object must have length to determine if number of \
-    parts having specified length(s) can be retrieved
-    >>> list(parts(iter([1,2,3,4,5,6,7]), 1))
+parts having specified length(s) can be retrieved
+    >>> list(parts(iter([1, 2, 3, 4, 5, 6, 7]), 1))
     Traceback (most recent call last):
       ...
     TypeError: object must have length to determine part lengths from number parameter
-    >>> list(parts([1,2,3,4,5,6], 1.2))
+    >>> list(parts([1, 2, 3, 4, 5, 6], 1.2))
     Traceback (most recent call last):
       ...
     TypeError: number parameter must be an integer
-    >>> list(parts([1,2,3,4,5,6], length=1.23))
+    >>> list(parts([1, 2, 3, 4, 5, 6], length=1.23))
     Traceback (most recent call last):
       ...
     TypeError: length parameter must be an integer or iterable of integers
-    >>> list(parts([1,2,3,4,5,6], length=[1.23]))
+    >>> list(parts([1, 2, 3, 4, 5, 6], length=[1.23]))
     Traceback (most recent call last):
       ...
     TypeError: length parameter must be an integer or iterable of integers
-    >>> list(parts([1,2,3,4,5,6], 2, length=[1,2,3]))
+    >>> list(parts([1, 2, 3, 4, 5, 6], 2, length=[1, 2, 3]))
     Traceback (most recent call last):
       ...
     ValueError: number parameter does not match number of specified part lengths
-    >>> list(parts([1,2,3,4,5,6,7], number=3, length=2))
+    >>> list(parts([1, 2, 3, 4, 5, 6, 7], number=3, length=2))
     Traceback (most recent call last):
       ...
     ValueError: cannot retrieve 3 parts from object given part length parameter of 2
-    >>> list(parts([1,2,3], length=4))
+    >>> list(parts([1, 2, 3], length=4))
     [[1, 2, 3]]
-    >>> list(parts([1,2,3], number=2, length=[1,2]))
+    >>> list(parts([1, 2, 3], number=2, length=[1, 2]))
     [[1], [2, 3]]
-    >>> list(parts([1,2,3], number=3, length=[1,2,3]))
+    >>> list(parts([1, 2, 3], number=3, length=[1, 2, 3]))
     Traceback (most recent call last):
       ...
     ValueError: object has too few items to retrieve parts having specified part lengths
-    >>> list(parts([1,2,3]))
+    >>> list(parts([1, 2, 3]))
     Traceback (most recent call last):
       ...
     ValueError: missing number of parts parameter and part length(s) parameter
-    >>> list(parts([1,2,3], length=[4]))
+    >>> list(parts([1, 2, 3], length=[4]))
     [[1, 2, 3]]
-    >>> list(parts([1,2,3], length=[3, 1]))
+    >>> list(parts([1, 2, 3], length=[3, 1]))
     Traceback (most recent call last):
       ...
     ValueError: object has too few items to retrieve parts having specified part lengths
-    >>> list(parts([1,2,3], number=2, length=[1,1]))
+    >>> list(parts([1, 2, 3], number=2, length=[1, 1]))
     Traceback (most recent call last):
       ...
     ValueError: object has too many items to retrieve parts having specified part lengths
-    >>> list(parts([1,2,3], number=1, length=[4]))
+    >>> list(parts([1, 2, 3], number=1, length=[4]))
     [[1, 2, 3]]
-    >>> list(parts([1,2,3], number=2, length=[3, 1]))
+    >>> list(parts([1, 2, 3], number=2, length=[3, 1]))
     Traceback (most recent call last):
       ...
     ValueError: object has too few items to retrieve parts having specified part lengths
-    >>> list(parts([1,2,3], length=[1,1,1,1]))
+    >>> list(parts([1, 2, 3], length=[1, 1, 1, 1]))
     Traceback (most recent call last):
       ...
     ValueError: object has too few items to retrieve parts having specified part lengths
-    >>> list(parts([1,2,3], number=1, length=[1.2]))
+    >>> list(parts([1, 2, 3], number=1, length=[1.2]))
     Traceback (most recent call last):
       ...
     TypeError: length parameter must be an integer or list of integers
-    >>> isinstance(next(parts([1,2,3,4,5], length=2)), list)
+
+    The type of input objects (for built-in types) is preserved in the output.
+
+    >>> isinstance(next(parts([1, 2, 3, 4, 5], length=2)), list)
     True
-    >>> isinstance(next(parts([1,2,3,4,5], length=[2, 2, 1])), list)
+    >>> isinstance(next(parts([1, 2, 3, 4, 5], length=[2, 2, 1])), list)
     True
-    >>> isinstance(next(parts([1,2,3,4,5], number=2)), list)
+    >>> isinstance(next(parts([1, 2, 3, 4, 5], number=2)), list)
     True
-    >>> isinstance(next(parts([1,2,3,4], number=2, length=2)), list)
+    >>> isinstance(next(parts([1, 2, 3, 4], number=2, length=2)), list)
     True
-    >>> isinstance(next(parts([1,2,3,4], number=2, length=[2, 2])), list)
+    >>> isinstance(next(parts([1, 2, 3, 4], number=2, length=[2, 2])), list)
     True
-    >>> isinstance(next(parts((1,2,3,4,5), length=2)), tuple)
+    >>> isinstance(next(parts((1, 2, 3, 4, 5), length=2)), tuple)
     True
-    >>> isinstance(next(parts((1,2,3,4,5), length=[2, 2, 1])), tuple)
+    >>> isinstance(next(parts((1, 2, 3, 4, 5), length=[2, 2, 1])), tuple)
     True
-    >>> isinstance(next(parts((1,2,3,4,5), number=2)), tuple)
+    >>> isinstance(next(parts((1, 2, 3, 4, 5), number=2)), tuple)
     True
-    >>> isinstance(next(parts((1,2,3,4), number=2, length=2)), tuple)
+    >>> isinstance(next(parts((1, 2, 3, 4), number=2, length=2)), tuple)
     True
-    >>> isinstance(next(parts((1,2,3,4), number=2, length=[2, 2])), tuple)
+    >>> isinstance(next(parts((1, 2, 3, 4), number=2, length=[2, 2])), tuple)
     True
     >>> isinstance(next(parts("abc", length=2)), str)
     True
@@ -195,25 +211,25 @@ def parts(xs, number=None, length=None):
     True
     >>> isinstance(next(parts("abcd", number=2, length=[2, 2])), str)
     True
-    >>> isinstance(next(parts(bytes([1,2,3,4,5]), length=2)), bytes)
+    >>> isinstance(next(parts(bytes([1, 2, 3, 4, 5]), length=2)), bytes)
     True
-    >>> isinstance(next(parts(bytes([1,2,3,4,5]), length=[2, 2, 1])), bytes)
+    >>> isinstance(next(parts(bytes([1, 2, 3, 4, 5]), length=[2, 2, 1])), bytes)
     True
-    >>> isinstance(next(parts(bytes([1,2,3,4,5]), number=2)), bytes)
+    >>> isinstance(next(parts(bytes([1, 2, 3, 4, 5]), number=2)), bytes)
     True
-    >>> isinstance(next(parts(bytes([1,2,3,4]), number=2, length=2)), bytes)
+    >>> isinstance(next(parts(bytes([1, 2, 3, 4]), number=2, length=2)), bytes)
     True
-    >>> isinstance(next(parts(bytes([1,2,3,4]), number=2, length=[2, 2])), bytes)
+    >>> isinstance(next(parts(bytes([1, 2, 3, 4]), number=2, length=[2, 2])), bytes)
     True
-    >>> isinstance(next(parts(bytearray([1,2,3,4,5]), length=2)), bytearray)
+    >>> isinstance(next(parts(bytearray([1, 2, 3, 4, 5]), length=2)), bytearray)
     True
-    >>> isinstance(next(parts(bytearray([1,2,3,4,5]), length=[2, 2, 1])), bytearray)
+    >>> isinstance(next(parts(bytearray([1, 2, 3, 4, 5]), length=[2, 2, 1])), bytearray)
     True
-    >>> isinstance(next(parts(bytearray([1,2,3,4,5]), number=2)), bytearray)
+    >>> isinstance(next(parts(bytearray([1, 2, 3, 4, 5]), number=2)), bytearray)
     True
-    >>> isinstance(next(parts(bytearray([1,2,3,4]), number=2, length=2)), bytearray)
+    >>> isinstance(next(parts(bytearray([1, 2, 3, 4]), number=2, length=2)), bytearray)
     True
-    >>> isinstance(next(parts(bytearray([1,2,3,4]), number=2, length=[2, 2])), bytearray)
+    >>> isinstance(next(parts(bytearray([1, 2, 3, 4]), number=2, length=[2, 2])), bytearray)
     True
     >>> isinstance(next(parts(range(0, 10), length=2)), range)
     True
@@ -225,8 +241,11 @@ def parts(xs, number=None, length=None):
     True
     >>> isinstance(next(parts(range(0, 4), number=2, length=[2, 2])), range)
     True
-    >>> isinstance(next(parts(iter([1,2,3,4]), length=2)), Iterable)
+    >>> isinstance(next(parts(iter([1, 2, 3, 4]), length=2)), Iterable)
     True
+
+    Iterable inputs yield iterable outputs when possible.
+
     >>> def iterable():
     ...     for i in range(10):
     ...         yield i
@@ -243,50 +262,56 @@ def parts(xs, number=None, length=None):
     Traceback (most recent call last):
       ...
     TypeError: object must have length to determine if number of \
-    parts having specified length(s) can be retrieved
+parts having specified length(s) can be retrieved
     >>> ps = parts(iterable(), number=2, length=[2, 2])
     >>> isinstance(next(ps), Iterable) # doctest: +NORMALIZE_WHITESPACE
     Traceback (most recent call last):
       ...
     TypeError: object must have length to determine if number of \
-    parts having specified length(s) can be retrieved
+parts having specified length(s) can be retrieved
     >>> not isinstance((next(parts(iterable(), length=2))), list)
     True
     >>> list(parts(123, length=2))
     Traceback (most recent call last):
       ...
     TypeError: object does not support retrieval of slices
-    >>> class wrap():
+
+    The type of input sequence objects is preserved in the output.
+
+    >>> class wrap:
     ...     def __init__(self, xs): self.xs = xs
     ...     def __len__(self): return len(self.xs)
     ...     def __getitem__(self, key): return wrap(self.xs[key])
     ...     def __repr__(self): return 'wrap(' + str(self.xs) + ')'
-    >>> isinstance(next(parts(wrap([1,2,3,4]), number=2)), wrap)
+    >>> isinstance(next(parts(wrap([1, 2, 3, 4]), number=2)), wrap)
     True
-    >>> list(parts(wrap([1,2,3,4]), number=2))
+    >>> list(parts(wrap([1, 2, 3, 4]), number=2))
     [wrap([1, 2]), wrap([3, 4])]
-    >>> list(parts(wrap([1,2,3,4]), length=2))
+    >>> list(parts(wrap([1, 2, 3, 4]), length=2))
     [wrap([1, 2]), wrap([3, 4])]
-    >>> list(parts(wrap([1,2,3,4]), length=[2, 2]))
+    >>> list(parts(wrap([1, 2, 3, 4]), length=[2, 2]))
     [wrap([1, 2]), wrap([3, 4])]
-    >>> list(parts(wrap([1,2,3,4]), number=2, length=2))
+    >>> list(parts(wrap([1, 2, 3, 4]), number=2, length=2))
     [wrap([1, 2]), wrap([3, 4])]
-    >>> list(parts(wrap([1,2,3,4]), number=2, length=[2, 2]))
+    >>> list(parts(wrap([1, 2, 3, 4]), number=2, length=[2, 2]))
     [wrap([1, 2]), wrap([3, 4])]
+
+    The type of input objects derived from a sequence type is preserved in the output.
+
     >>> class inherit(tuple):
     ...     def __getitem__(self, key): return inherit(tuple(self)[key])
     ...     def __repr__(self): return 'inherit' + str(tuple(self))
-    >>> isinstance(next(parts(inherit([1,2,3,4]), 2)), inherit)
+    >>> isinstance(next(parts(inherit([1, 2, 3, 4]), 2)), inherit)
     True
-    >>> list(parts(inherit([1,2,3,4]), number=2))
+    >>> list(parts(inherit([1, 2, 3, 4]), number=2))
     [inherit(1, 2), inherit(3, 4)]
-    >>> list(parts(inherit([1,2,3,4]), length=2))
+    >>> list(parts(inherit([1, 2, 3, 4]), length=2))
     [inherit(1, 2), inherit(3, 4)]
-    >>> list(parts(inherit([1,2,3,4]), length=[2,2]))
+    >>> list(parts(inherit([1, 2, 3, 4]), length=[2, 2]))
     [inherit(1, 2), inherit(3, 4)]
-    >>> list(parts(inherit([1,2,3,4]), number=2, length=2))
+    >>> list(parts(inherit([1, 2, 3, 4]), number=2, length=2))
     [inherit(1, 2), inherit(3, 4)]
-    >>> list(parts(inherit([1,2,3,4]), number=2, length=[2,2]))
+    >>> list(parts(inherit([1, 2, 3, 4]), number=2, length=[2, 2]))
     [inherit(1, 2), inherit(3, 4)]
     """
     if number is not None and not isinstance(number, int):
@@ -317,10 +342,9 @@ def parts(xs, number=None, length=None):
             if number == 0:
                 yield xs[i:]
                 break
-            else:
-                yield xs[i:i + length]
-                i += length
-                length = (len_ - i) // number
+            yield xs[i:i + length]
+            i += length
+            length = (len_ - i) // number
 
     elif number is None and length is not None:
         if isinstance(length, int):
@@ -356,7 +380,7 @@ def parts(xs, number=None, length=None):
                     (part, empty) = _empty(part)
                     if empty:
                         raise ValueError(
-                            "object has too few items to retrieve parts having " +\
+                            "object has too few items to retrieve parts having " + \
                             "specified part lengths"
                         )
                     yield part
@@ -368,7 +392,7 @@ def parts(xs, number=None, length=None):
             len_ = len(xs)
         except:
             raise TypeError(
-                "object must have length to determine if number of " +\
+                "object must have length to determine if number of " + \
                 "parts having specified length(s) can be retrieved"
             ) from None
 
@@ -376,12 +400,12 @@ def parts(xs, number=None, length=None):
             length = max(1, length)
             if len_ > (length * number) or len_ <= (length * (number - 1)):
                 raise ValueError(
-                    "cannot retrieve " + str(number) + " parts from object " +\
+                    "cannot retrieve " + str(number) + " parts from object " + \
                     "given part length parameter of " + str(length)
                 )
             for i in range(0, len_, length): # Yield parts of specified length.
                 yield xs[i:i + length]
-        elif (not isinstance(length, list)) or\
+        elif (not isinstance(length, list)) or \
              (not all(isinstance(l, int) for l in length)):
             raise TypeError(
                 "length parameter must be an integer or list of integers"
@@ -393,12 +417,12 @@ def parts(xs, number=None, length=None):
                 )
             elif len_ <= sum(length[:-1]):
                 raise ValueError(
-                    "object has too few items to retrieve parts having " +\
+                    "object has too few items to retrieve parts having " + \
                     "specified part lengths"
                 )
             elif len_ > sum(length):
                 raise ValueError(
-                    "object has too many items to retrieve parts having " +\
+                    "object has too many items to retrieve parts having " + \
                     "specified part lengths"
                 )
             else:
